<?php

/**
 * BuddyServ IRC Services
 * 
 * ServiceClient object used by all pseudo
 * service clients. Provides common functionality
 * to the IRC Server.
 * 
 * @name ServiceClient
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.serviceclient.php 331 2009-11-22 04:21:18Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class serviceclient
{
	public $irc;
	public $username;
	public $host;
	public $name;
	public $nick;
	public $modes;
	public $help_list = array();
	
	/**
	 * Initialize the generic model
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	public function initialize($nick, $username, $host, $name, $modes)
	{
		global $irc;
		
		$this->nick = $nick;
		$this->host = $host;
		$this->username = $username;
		$this->name = $name;
		$this->modes = $modes;
		$this->irc = &$irc;
		
		$this->load_help();
	}
	
	/**
	 * Introduce the client to the network
	 * 
	 * @return void
	 */
	public function introduce()
	{
		global $me;
		
		if (!$this->nick)
			die('Call to introduce() on non initialized service client');
			
		$this->irc->send(sprintf(INTRODUCE, $this->nick, 
			1, 
			time(), 
			$this->modes, 
			$this->username, 
			$this->host, 
			$me->name, 
			0, 
			$me->name, 
			$this->name));
	}
	
	/**
	 * Sends an IRC Notice
	 * 
	 * @param string $target
	 * @param string $message
	 * @return void
	 */
	public function notice($target, $message)
	{
		$this->irc->send(sprintf(NOTICE, $this->nick, $target, $message));
	}
	
	/**
	 * Sets a channels modes
	 * 
	 * @param string $target
	 * @param string $modes
	 * @return unknown_type
	 */
	public function chanMode($target, $modes)
	{
		$this->irc->send(sprintf(MODE, $this->nick, $target, $modes));
		handler::cMode(array(null, false, $target, $modes));
	}
	
	/**
	 * Show an 'Access Denied' string to a client.
	 * 
	 * @param string $target
	 * @return void
	 */
	public function denied($target)
	{
		self::notice($target, 'Access denied.');
	}
	
	/**
	 * Sends an operwall
	 * 
	 * @param string $message
	 * @return void
	 */
	public function operwall($message)
	{
		$this->irc->send(sprintf(OPERWALL, $this->nick, $message));
	}
	
	/**
	 * Join a channel
	 * 
	 * @param string $channel
	 * @return void
	 */
	public function join($channel)
	{
		global $me;
		$this->irc->send(sprintf(SJOIN, $me->name, time(), $channel, '+', $this->nick));
	}
	
	/**
	 * Set a channel mode. Legacy place holder.
	 * 
	 * @param string $channel
	 * @param string $modes
	 * @return void
	 */
	public function cmode($channel,$modes)
	{
		self::chanMode($channel, $modes);
	}
	
	/**
	 * Kick a user from a channel
	 * 
	 * @param string $channel
	 * @param string $who
	 * @param string $reason
	 * @return void
	 */
	public function kick($channel, $who, $reason='No reason specified.')
	{
		global $me;
		$this->irc->send(sprintf(KICK, $this->nick, $channel, $who, $reason));
		
		$userObj = user::getuser($who);
		$userObj->removeChannel($channel);
		
		$chanObj = channel::get($channel);
		$chanObj->deluser($who);
		
		if (strcasecmp($channel, $me->user_channel))
			idle::remove($who, true);
	}
	
	/**
	 * Change a channel topic
	 * 
	 * @param string $channel
	 * @param string $topic
	 * @return void
	 */
	public function topic($channel, $topic)
	{
		$this->irc->send(sprintf(TOPIC, $this->nick, $channel, $topic));
	}
	
	/**
	 * Part a channel
	 * 
	 * @param string $channel
	 * @param string $reason
	 * @return void
	 */
	public function part($channel, $reason=false)
	{
		$this->irc->send(sprintf(PART, $this->nick, $channel, ($reason ? $reason : '')));
	}
	
	/**
	 * Quits the current service client
	 * 
	 * @param string $reason
	 * @return void
	 */
	public function quit($reason='BuddyServ by Lynk')
	{
		$this->irc->send(sprintf(QUIT, $this->nick, $reason));
	}
			
	/**
	 * Send a PRIVMSG
	 * 
	 * @param string $target
	 * @param string $message
	 * @return void
	 */
	public function msg($target, $message)
	{
		$this->irc->send(sprintf(MSG, $this->nick, $target, $message));
	}
	
	/**
	 * Load the helpfiles.
	 * 
	 * @return void
	 */
	public function load_help()
	{
		$this->help_list = array();
		
		if (is_dir('help/'.strtolower($this->nick)))
		{
			/* check if we have a valid help directory, if so, read the file listing */
			if ($handle = opendir('help/'.strtolower($this->nick)))
			{
			//	echo '* Loading '.$this->nick.' helpfiles: ';
				
				while (($file = readdir($handle)) !== false)
				{
					/* we never really want to recursively search, and ../ is bad */
					if (filetype('help/'.strtolower($this->nick).'/'.$file) == 'dir')
						continue;
						
					/* if the file is an empty placeholder dont include it */
					if (filesize('help/'.strtolower($this->nick).'/'.$file) == 0)
					{
						//echo '('.$file.'), ';
						continue;
					}
					
				//	echo $file.', ';
					
					/* open the file read only */
					$fhandle = fopen('help/'.strtolower($this->nick).'/'.$file, 'r');
					
					$buffer = fread($fhandle, filesize('help/'.strtolower($this->nick).'/'.$file));
					fclose($fhandle);
					
					/* Remove the file extension from the name  */
					$filestoreas = substr($file, 0, (strlen($file) - 4));
					
					$this->help_list[$filestoreas] = explode("\r\n", $buffer);
				}
				
				closedir($handle);
			}
		}
	}
	
	/**
	 * Handle help requests for the service client.
	 * 
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	public function help($nick, $arguments=false)
	{
		$user = user::getuser($nick);
		
		if (!$arguments)
		{
			if (array_key_exists('index', $this->help_list))
			{
				foreach ($this->help_list['index'] as $number => $line)
				{
					$this->notice($nick, $line);
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_UPLOADER))
			{
				if (array_key_exists('oindex', $this->help_list))
				{
					foreach ($this->help_list['oindex'] as $number => $line)
					{
						$this->notice($nick, $line);
					}
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_MODERATOR))
			{
				if (array_key_exists('aindex', $this->help_list))
				{
					foreach ($this->help_list['aindex'] as $number => $line)
					{
						$this->notice($nick, $line);
					}
				}
			}
			
		}
		
		else 
		{	
			if (count($arguments) > 1)
			{
				$type = strtolower(implode('_', $arguments));
			}
			
			else
			{
				$type = strtolower($arguments[0]);
			}
				
			if (array_key_exists($type, $this->help_list))
			{
				foreach ($this->help_list[$type] as $number => $line)
				{
					$this->notice($nick, $line);
				}
				
				return;
			}
				
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_UPLOADER))
			{
				$tkey = 'o'.$type;
				
				if (array_key_exists($tkey, $this->help_list))
				{
					foreach ($this->help_list[$tkey] as $number => $line)
					{
						$this->notice($nick, $line);
					}
					
					return;
				}
			}
			
			if ((($user->modes & UMODE_REGISTERED) > 0) and ($user->level > SITE_ADMIN))
			{
				$tkey = 'a'.$type;
				
				if (array_key_exists($tkey, $this->help_list))
				{
					foreach ($this->help_list[$tkey] as $number => $line)
					{
						$this->notice($nick, $line);
					}
					
					return;
				}
			}
			
			/* unknown topix */
			$this->notice($nick, 'No such help topic.');
		}
	}
	
}

?>
	